/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using GLib;
using Config;

namespace Unity.VideoLens {

  static const string BUS_NAME = "net.launchpad.scope.RemoteVideos";
  Unity.Scope scope;

  public static int main (string[] args)
  {
    GLib.Environment.set_prgname ("unity-remote-video-scope");

    /* Sort up locale to get translations but also sorting and
     * punctuation right */
    GLib.Intl.textdomain (Config.PACKAGE);
    GLib.Intl.bindtextdomain (Config.PACKAGE, Config.LOCALEDIR);
    GLib.Intl.bind_textdomain_codeset (Config.PACKAGE, "UTF-8");
    GLib.Intl.setlocale(GLib.LocaleCategory.ALL, "");

    /* Workaround for https://bugzilla.gnome.org/show_bug.cgi?id=640714
     * GApplication.register() call owns our DBus name in a sync manner
     * making it race against GDBus' worker thread to export our
     * objects on the bus before/after owning our name and receiving
     * method calls on our objects (which may not yet be up!)*/
    if (Utils.dbus_name_has_owner (BUS_NAME))
    {
      print ("Another instance of the Unity Videos Lens " +
             "already appears to be running.\nBailing out.\n");
      return 2;
    }

    /* Now register our DBus objects *before* acquiring the name!
     * See above for reasons */
    scope = new RemoteVideoScope ();

    /* Use GApplication directly for single instance app functionality */
    Application? app = new Application (BUS_NAME, ApplicationFlags.IS_SERVICE);
    try
    {
      app.register ();
    }
    catch (Error e)
    {
      /* FIXME: We get this error if another daemon is already running,
       * but it uses a generic error so we can't detect this reliably... */
      print ("Failed to start video lens daemon: %s\n", e.message);
      return 1;
    }

    if (app.get_is_remote ())
    {
      print ("Another instance of the Unity Video Lens Daemon " +
             "already appears to be running.\nBailing out.\n");
      return 2;
    }

    /* Hold()ing the app makes sure the GApplication doesn't exit */
    app.hold ();
    return app.run ();
  }

} /* namespace */
